#! /bin/bash
#======================================================================
#
# fpp_wrapper.sh <FPP command> <wrapping mode> <FPP arguments>
#
# Wrapped FPP command for optional additional PSyclone processing of
# source code
#
#======================================================================
# History : 4.0 ! 2021/2022 (S. Mueller) Original version
#----------------------------------------------------------------------

#----------------------------------------------------------------------
# NEMO 4.0 , NEMO Consortium (2022)
# $Id$ 
# Software governed by the CeCILL license (see ./LICENSE)
#----------------------------------------------------------------------

# Path to PSyclone transformation script
PSyclone_TS=../mk_psyclone/psyclone_transformation_passthrough.py

FPP=$1; shift
# Extended pre-processing mode
MODE=$1; shift
# Original arguments
FPP_ARG=$@; shift $(($# - 1))
#
# PSyclone pre-processing of NEMO source-code files
if [ ${MODE} == 'PSyclone' ]; then
    # Filename of file to be processed
    fname=$1
    # Call wrapped preprocessor, store in temporary file
    $FPP -o ${fname}.preprocessed.$$ ${FPP_ARG}
    # PSyclone pre-processing
    # PSyclone processing, unless ACC directives present
    if `grep -q '!\$ACC' ${fname}`; then
	cat ${fname}.preprocessed.$$
    else
        case ${fname} in
	# Do NOT process some files that do not pass the PSyclone pass-through test (see also PSyclone file examples/nemo/scripts/process_nemo.sh)
	    ( obs_utils.[fF]90 | par_kind.[fF]90 | storng.[fF]90 | diurnal_bulk.[fF]90 | timing.[fF]90 | obs_fbm.[fF]90 | sbccpl.F90 ) cat ${fname}.preprocessed.$$ ;;
	# Special case for mocsy_mainmod.F90
	    ( mocsy_mainmod.[fF]90 ) psyclone -api nemo -l output -s ${COMPIL_DIR}/${PSyclone_TS} -oalg /dev/null ${fname}.preprocessed.$$ | tail -n +2 | sed -e 's/ :: onehalf/, parameter :: onehalf=0.5d0/' -e 's/ :: two/, parameter :: two=2.0d0/' -e 's/ :: three/, parameter :: three=3.0d0/' ;;
	# Special case for trcini_medusa.F90 and trcrst_medusa.F90
	    ( trcrst_medusa.[fF]90 | trcini_medusa.[fF]90 ) sed -i -e "s/element:'a10/element:',a10/" ${fname}.preprocessed.$$ ; psyclone -api nemo -l output -s ${COMPIL_DIR}/${PSyclone_TS} -oalg /dev/null ${fname}.preprocessed.$$ | tail -n +2 ;;
	# PSyclone processing
            ( *.[fF]90 ) psyclone -api nemo -l output -s ${COMPIL_DIR}/${PSyclone_TS} -oalg /dev/null ${fname}.preprocessed.$$ | tail -n +2 ;;
	# Do nothing with other NEMO source-code files
	    ( * ) cat ${fname}.preprocessed.$$ ;;
        esac;
    fi
    # Remove temporary file
    rm ${fname}.preprocessed.$$
else
    # Transparent call of wrapped pre-processor
    $FPP ${FPP_ARG}
fi
